package com.yunzhi.shigongli.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.shigongli.common.*;
import com.yunzhi.shigongli.entity.TaHotelAccount;
import com.yunzhi.shigongli.entity.TaPerson;
import com.yunzhi.shigongli.entity.TaShop;
import com.yunzhi.shigongli.service.ITaHotelAccountService;
import com.yunzhi.shigongli.service.ITaMiniappQrcodeService;
import com.yunzhi.shigongli.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import com.yunzhi.shigongli.service.ITaHotelService;
import com.yunzhi.shigongli.entity.TaHotel;

import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * <p>
    * 民宿 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */

@Api(tags = "民宿")
@RestController
@RequestMapping("/")
public class TaHotelController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaHotelController.class);

    @Autowired
    public ITaHotelService iTaHotelService;

    @Autowired
    public ITaHotelAccountService iTaHotelAccountService;

    @Autowired
    public ITaMiniappQrcodeService iTaMiniappQrcodeService;

    @Value("${sysBiz.miniapp.indexPage}")
    public String indexPage;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{plat}/taHotel",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = HotelListVO.class)
    public ResponseBean taHotelList(@ApiParam(value = "客户端", allowableValues = "wx,admin") @PathVariable String plat,
                                    @ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                    @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                    @ApiParam("名称") @RequestParam(value = "hotelName",required = false) String hotelName) throws Exception{


        IPage<HotelListVO> pg = new Page<>(pageNum, pageSize);
        IPage<HotelListVO> result = iTaHotelService.getVOListBy(pg, hotelName);
        return ResponseBean.success(result);
    }

    /**
     * 导出民宿列表
     * @return
     */
    @RequestMapping(value="/{plat}/taHotel/export",method= RequestMethod.GET)
    @ApiOperation(value="导出民宿列表", notes = "导出民宿列表", httpMethod = "GET", response = HotelListVO.class)
    public void exportHotelList(@ApiParam("名称") @RequestParam(value = "hotelName",required = false) String hotelName,
                                HttpServletResponse response) throws Exception{

        IPage<HotelListVO> result = iTaHotelService.getVOListBy(null, hotelName);

        ExcelUtils.flush(response, HotelListVO.class, result.getRecords(), "民宿列表");
    }


    /**
     * 民宿推广收入列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/taHotel/commission",method= RequestMethod.GET)
    @ApiOperation(value="民宿推广收入列表", notes = "民宿推广收入列表", httpMethod = "GET", response = HotelCommissionVO.class)
    public ResponseBean taHotelComm(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                    @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                    @ApiParam("店铺名称") @RequestParam(value = "hotelName", required = false) String hotelName,
                                    @ApiParam(value = "开始时间", example = "2021-09-01") @RequestParam(value = "start", required = false) String start,
                                    @ApiParam(value = "结束时间", example = "2021-09-30") @RequestParam(value = "end", required = false) String end) throws Exception{


        IPage<HotelCommissionVO> pg = new Page<>(pageNum, pageSize);
        LocalDateTime startDate = null;
        LocalDateTime endDate = null;
        if (!StringUtils.isEmpty(start)) {
            startDate = DateUtils.from(start + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
        }
        if (!StringUtils.isEmpty(end)) {
            endDate = DateUtils.from(end + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }

        IPage<HotelCommissionVO> result = iTaHotelService.getCommissionList(pg, hotelName, startDate, endDate);
        return ResponseBean.success(result);
    }

    /**
     * 民宿推广收入列表导出
     * @return
     */
    @RequestMapping(value="/admin/taHotel/commission/export",method= RequestMethod.GET)
    @ApiOperation(value="民宿推广收入列表", notes = "民宿推广收入列表导出", httpMethod = "GET", response = HotelCommissionVO.class)
    public void taHotelExport(@ApiParam("店铺名称") @RequestParam(value = "hotelName", required = false) String hotelName,
                              @ApiParam(value = "开始时间", example = "2021-09-01") @RequestParam(value = "start", required = false) String start,
                              @ApiParam(value = "结束时间", example = "2021-09-30") @RequestParam(value = "end", required = false) String end,
                              HttpServletResponse response) throws Exception{

        LocalDateTime startDate = null;
        LocalDateTime endDate = null;
        if (!StringUtils.isEmpty(start)) {
            startDate = DateUtils.from(start + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
        }
        if (!StringUtils.isEmpty(end)) {
            endDate = DateUtils.from(end + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }

        IPage<HotelCommissionVO> result = iTaHotelService.getCommissionList(null, hotelName, startDate, endDate);

        ExcelUtils.flush(response, HotelCommissionVO.class, result.getRecords(), "民宿推广收入");
    }

    /**
     * 保存对象
     * @param taHotel 实体对象
     * @return
     */
    @RequestMapping(value="/admin/taHotel",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taHotelAdd(@ApiParam("保存内容") @RequestBody TaHotel taHotel) throws Exception{

        if (null == taHotel.getStatus()) {
            taHotel.setStatus(Constants.STATUS_NORMAL);
        }

        if (iTaHotelService.createNew(taHotel)){
            return ResponseBean.success(taHotel);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/taHotel/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taHotelDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        if(iTaHotelService.removeLogicById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taHotel 实体对象
     * @return
     */
    @RequestMapping(value="/admin/taHotel/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taHotelUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody TaHotel taHotel) throws Exception{

//        boolean exists = iTaHotelService.isExisted(TaHotel::getHotelName,
//                taHotel.getHotelName(),
//                TaHotel::getHotelId,
//                taHotel.getHotelId());
//        if (exists) {
//            return ResponseBean.error("民宿名称已存在", ResponseBean.ERROR_ILLEGAL_PARAMS);
//        }

        if (iTaHotelService.updateById(taHotel)){
            return ResponseBean.success(iTaHotelService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/taHotel/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taHotelGet(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(iTaHotelService.getById(id));
    }


    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/wx/hotel/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getWxDetail(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(iTaHotelService.getById(id));
    }

    /**
     * 民宿小程序首页
     */
    @RequestMapping(value="/wx/hotel/manage/index",method= RequestMethod.GET)
    @ApiOperation(value="民宿小程序首页", notes = "民宿小程序首页", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getWxManageIndex() throws Exception{
        TaPerson taPerson = getCurrentPerson();
        if (!Constants.TARGET_HOTEL.equals(taPerson.getPersonRole())) {
            return ResponseBean.error("请先进行人员身份绑定", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        // 获取所有的民宿信息
        List<TaHotel> hotelList = iTaHotelService.getListByPerson(taPerson);
        if (null == hotelList || hotelList.size() < 1) {
            return ResponseBean.error("数据异常, 未找到关联民宿", ResponseBean.ERROR_UNAVAILABLE);
        }

        // 默认获取第一个店铺信息
        TaHotel taHotel = hotelList.get(0);
        TaHotelAccount taHotelAccount = iTaHotelAccountService.getAccountInfoBy(taHotel.getHotelId());

        Map<String, Object> result = new HashMap<>();
        result.put("hotelList", hotelList);
        result.put("hotel", taHotel);
        result.put("account", taHotelAccount);

        return ResponseBean.success(result);
    }

    /**
     * 民宿小程序首页
     */
    @RequestMapping(value="/wx/hotel/manage/{id}",method= RequestMethod.GET)
    @ApiOperation(value="民宿小程序首页", notes = "民宿小程序首页", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getWxManageHotel(@ApiParam("民宿ID") @PathVariable String id) throws Exception{
        TaPerson taPerson = getCurrentPerson();
        if (!Constants.TARGET_HOTEL.equals(taPerson.getPersonRole())) {
            return ResponseBean.error("请先进行人员身份绑定", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        // 获取所有的民宿信息
        List<TaHotel> hotelList = iTaHotelService.getListByPerson(taPerson);
        if (null == hotelList || hotelList.size() < 1) {
            return ResponseBean.error("数据异常, 未找到关联民宿", ResponseBean.ERROR_UNAVAILABLE);
        }

        Optional<TaHotel> first = hotelList.stream().filter(x -> x.getHotelId().equals(id)).findFirst();
        if (!first.isPresent()) {
            return ResponseBean.error("数据异常, 未找到关联民宿", ResponseBean.ERROR_UNAVAILABLE);
        }

        // 默认获取第一个店铺信息
        TaHotel taHotel = first.get();
        TaHotelAccount taHotelAccount = iTaHotelAccountService.getAccountInfoBy(taHotel.getHotelId());

        Map<String, Object> result = new HashMap<>();
        result.put("hotel", taHotel);
        result.put("account", taHotelAccount);

        return ResponseBean.success(result);
    }

    /**
     * 民宿查询统计
     */
    @RequestMapping(value="/admin/hotel/query/list",method= RequestMethod.GET)
    @ApiOperation(value="民宿查询统计", notes = "民宿查询统计", httpMethod = "GET", response = HotelQueryVO.class)
    public ResponseBean queryHotel(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                   @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                   @ApiParam("名称") @RequestParam(value = "hotelName",required = false) String hotelName,
                                   @ApiParam(value = "开始日期", example = "2021-09-01") @RequestParam(value = "start",required = false) String start,
                                   @ApiParam(value = "结束日期", example = "2021-09-30") @RequestParam(value = "end",required = false) String end) throws Exception {
        LocalDateTime startDate = null, endDate = null;

        if (!StringUtils.isEmpty(start)) {
            startDate = DateUtils.from(start + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
        }
        if (!StringUtils.isEmpty(end)) {
            endDate = DateUtils.from(end + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }

        IPage<HotelQueryVO> pg = new Page<>(pageNum, pageSize);
        IPage<HotelQueryVO> result = iTaHotelService.queryHotel(pg, hotelName, startDate, endDate);
        return ResponseBean.success(result);
    }


    /**
     * 民宿查询统计导出
     */
    @RequestMapping(value="/admin/hotel/query/export",method= RequestMethod.GET)
    @ApiOperation(value="民宿查询统计导出", notes = "民宿查询统计导出", httpMethod = "GET", response = HotelQueryVO.class)
    public void exportQueryHotel(@ApiParam("名称") @RequestParam(value = "hotelName",required = false) String hotelName,
                                 @ApiParam(value = "开始日期", example = "2021-09-01") @RequestParam(value = "start",required = false) String start,
                                 @ApiParam(value = "结束日期", example = "2021-09-30") @RequestParam(value = "end",required = false) String end,
                                 HttpServletResponse response) throws Exception {
        LocalDateTime startDate = null, endDate = null;

        if (!StringUtils.isEmpty(start)) {
            startDate = DateUtils.from(start + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
        }
        if (!StringUtils.isEmpty(end)) {
            endDate = DateUtils.from(end + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }
        IPage<HotelQueryVO> result = iTaHotelService.queryHotel(null, hotelName, startDate, endDate);

        ExcelUtils.flush(response, HotelQueryVO.class, result.getRecords(), "民宿查询统计");
    }

    @PostMapping("/admin/hotel/{id}/qrcode")
    @ApiOperation(value="民宿二维码", notes = "民宿二维码", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean generateQrCode(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        TaHotel taHotel = iTaHotelService.getById(id);
        if (null == taHotel || Constants.STATUS_DELETED == taHotel.getStatus()) {
            return ResponseBean.error("当前民宿信息不存在", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        QrCodeParamsVO qrCodeParamsVO = new QrCodeParamsVO();
        qrCodeParamsVO.setPage(indexPage);
        qrCodeParamsVO.setType(Constants.TARGET_HOTEL);
        qrCodeParamsVO.setScene(new HashMap<String, Object>(){{
            put("id", id);
        }});
        String qrCode = iTaMiniappQrcodeService.createQrCode(qrCodeParamsVO);

        return ResponseBean.success(new HashMap<String, Object>(){{
            put("qrCode", qrCode);
        }});
    }
}
