package com.yunzhi.shigongli.event;


import cn.binarywang.wx.miniapp.bean.WxMaSubscribeMessage;
import com.yunzhi.shigongli.common.Constants;
import com.yunzhi.shigongli.common.DateUtils;
import com.yunzhi.shigongli.common.WxUtils;
import com.yunzhi.shigongli.entity.TaMiniappMessageTpl;
import com.yunzhi.shigongli.mapper.TaMiniappMessageTplMapper;
import com.yunzhi.shigongli.service.ITaRoomOrderService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 发送消息
 */
@Slf4j
@Component
public class SendMessageTask {

    @Autowired
    WxUtils wxUtils;

    @Autowired
    ITaRoomOrderService iTaRoomOrderService;

    @Autowired
    TaMiniappMessageTplMapper messageTplMapper;

    /**
     * 离店消息
     */
    @Scheduled(cron = "0 0 11 * * ?")
    public void hotelCheckOut() {
        String today = DateUtils.toString(LocalDateTime.now(), "yyyy-MM-dd");

        TaMiniappMessageTpl messageTpl = messageTplMapper.selectByType(Constants.TPL_MESSAGE_HOTEL_CHECK_OUT);
        if (null == messageTpl) {
            return ;
        }

        List<Map<String, Object>> list = iTaRoomOrderService.getCheckoutList(today);

        if (null == list) {
            return ;
        }

        String templateId = messageTpl.getTemplateId();
        String remark = "请记得带好您的随身物品";

        for (Map<String, Object> item : list) {
            List<WxMaSubscribeMessage.Data> dataList = new ArrayList<>();
            dataList.add(new WxMaSubscribeMessage.Data("time1", String.format("%s 12:00", today)));
            dataList.add(new WxMaSubscribeMessage.Data("thing2", remark));

            String openid = item.get("openid").toString();
            String hotelId = item.get("hotelId").toString();
            String page = messageTpl.getPage() + "&id=" + hotelId;

            try {
                wxUtils.sendMessage(templateId, openid, page, dataList);
            } catch (Exception e) {
                e.printStackTrace();
                log.error("发送离店消息给游客失败: {}", e.getMessage());
            } finally {
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }

    }
}
