package com.yunzhi.shigongli.service.impl;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaSubscribeMessage;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.yunzhi.shigongli.common.Constants;
import com.yunzhi.shigongli.common.StringUtils;
import com.yunzhi.shigongli.common.WxUtils;
import com.yunzhi.shigongli.entity.*;
import com.yunzhi.shigongli.mapper.*;
import com.yunzhi.shigongli.service.ITaHotelWorkerService;

import com.yunzhi.shigongli.vo.HotelWorkerListVO;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 民宿员工 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */
@Slf4j
@Service
public class TaHotelWorkerServiceImpl extends BaseServiceImpl<TaHotelWorkerMapper, TaHotelWorker> implements ITaHotelWorkerService {

    @Autowired
    WxUtils wxUtils;

    @Autowired
    TaHotelWorkerMapper hotelWorkerMapper;

    @Autowired
    TaHotelMapper taHotelMapper;

    @Autowired
    TaRoomMapper taRoomMapper;

    @Autowired
    TaMiniappMessageTplMapper messageTplMapper;

    @Autowired
    TaPersonMapper taPersonMapper;

    @Override
    public IPage<HotelWorkerListVO> getPagedListBy(IPage<HotelWorkerListVO> pg, String hotelId) {
        return hotelWorkerMapper.getPagedListBy(pg, hotelId);
    }

    @Override
    public List<TaHotelWorker> getByPhone(String phone) {
        LambdaQueryWrapper<TaHotelWorker> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TaHotelWorker::getPhone, phone);
        queryWrapper.gt(TaHotelWorker::getStatus, Constants.STATUS_DELETED);

        return list(queryWrapper);
    }

    @Override
    public void updateByPerson(TaPerson taPerson) throws Exception {
        hotelWorkerMapper.updateByPerson(taPerson.getPersonId(), taPerson.getPhone());
    }

    @Override
    public void sendMessageBy(TaRoomOrder taRoomOrder) {
        TaHotel taHotel = taHotelMapper.selectById(taRoomOrder.getHotelId());
        List<TaHotelWorker> workerList = getListByHotel(taHotel.getHotelId());
        if (null == workerList || workerList.size() < 1) {
            return;
        }

        TaRoom taRoom = taRoomMapper.selectById(taRoomOrder.getRoomId());
        if (null == taRoom) {
            return;
        }

        TaMiniappMessageTpl messageTpl = messageTplMapper.selectByType(Constants.TPL_MESSAGE_HOTEL_CHECK_IN);
        if (null == messageTpl) {
            return ;
        }

        // 发送内容
        List<WxMaSubscribeMessage.Data> dataList = new ArrayList<>();
        String roomName = taRoom.getRoomName();
        if (null != roomName && roomName.length() > 10) {
            roomName = roomName.substring(0, 10) + "...";
        }
        dataList.add(new WxMaSubscribeMessage.Data("thing6", roomName));
        dataList.add(new WxMaSubscribeMessage.Data("date2", taRoomOrder.getStartDate()));

        String templateId = messageTpl.getTemplateId();
        String page = messageTpl.getPage()
                + "?roomId=" + taRoom.getRoomId()
                + "&hotelId=" + taRoom.getHotelId()
                + "&roomName=" + StringUtils.encodeURIComponent(taRoom.getRoomName());

        // 暂时先发送给所有店主, 1s 发送一个
        for (TaHotelWorker worker : workerList) {
            String toUser = taPersonMapper.getOpenid(worker.getPersonId());
            try {

                wxUtils.sendMessage(templateId, toUser, page, dataList);
            } catch (Exception e) {
                log.error("发送订阅消息给店主失败: {}", e.getMessage());
                e.printStackTrace();
            } finally {
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }

    }

    private List<TaHotelWorker> getListByHotel(String hotelId) {
        LambdaQueryWrapper<TaHotelWorker> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TaHotelWorker::getHotelId, hotelId);
        queryWrapper.gt(TaHotelWorker::getStatus, Constants.STATUS_DELETED);

        return list(queryWrapper);
    }

}
