package com.yunzhi.shigongli.service.impl;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaSubscribeMessage;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.yunzhi.shigongli.common.Constants;
import com.yunzhi.shigongli.common.StringUtils;
import com.yunzhi.shigongli.common.WxUtils;
import com.yunzhi.shigongli.entity.*;
import com.yunzhi.shigongli.mapper.*;
import com.yunzhi.shigongli.service.ITaVerifyTargetService;
import com.yunzhi.shigongli.vo.VerifyTargetVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.object.UpdatableSqlQuery;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 核销内容表 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */
@Slf4j
@Service
public class TaVerifyTargetServiceImpl extends BaseServiceImpl<TaVerifyTargetMapper, TaVerifyTarget> implements ITaVerifyTargetService {

    @Autowired
    TaOrderSubMapper taOrderSubMapper;

    @Autowired
    TaOrderCommissionMapper commissionMapper;

    @Autowired
    TaPersonRecommenderMapper recommenderMapper;

    @Autowired
    TaShopAccountMapper shopAccountMapper;

    @Autowired
    TaShopAccountLogMapper shopAccountLogMapper;

    @Autowired
    TaHotelAccountMapper hotelAccountMapper;

    @Autowired
    TaHotelAccountLogMapper hotelAccountLogMapper;

    @Autowired
    TaMiniappMessageTplMapper messageTplMapper;

    @Autowired
    WxUtils wxUtils;

    @Override
    public IPage<VerifyTargetVO> getWxPagedBy(IPage<VerifyTargetVO> pg, String personId, String shopId, String phone, Integer isVerified) {
        return baseMapper.getWxPagedBy(pg, personId, shopId, phone, isVerified);
    }

    @Override
    public int getCountBy(String orderId, String subOrderId, Integer isVerified) {
        return baseMapper.getCountBy(orderId, subOrderId, isVerified);
    }

    @Override
    public boolean removeBy(String orderId, String subOrderId) {
        UpdateWrapper<TaVerifyTarget> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("status", Constants.STATUS_DELETED);
        updateWrapper.eq("order_id", orderId);
        updateWrapper.eq(!StringUtils.isEmpty(subOrderId), "sub_order_id", subOrderId);

        return update(updateWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void verify(TaVerifyTarget taVerifyTarget) throws Exception {
        taVerifyTarget.setIsVerified(true);
        taVerifyTarget.setVerifiedDate(LocalDateTime.now());

        if (!updateById(taVerifyTarget)) {
            throw new Exception("核销失败, 请重试");
        }
    }

    @Async
    @Override
    public void sendVerifyResultNotifyToShop(TaVerifyTarget taVerifyTarget) {
        try {
            // 消息模板
            TaMiniappMessageTpl messageTpl = messageTplMapper.selectByType(Constants.TPL_MESSAGE_SHOP_VERIFY_RESULT);
            if (null == messageTpl) {
                return ;
            }

            String templateId = messageTpl.getTemplateId();
            String page = messageTpl.getPage();

            String orderId = taVerifyTarget.getOrderId();
            String shopId = taVerifyTarget.getShopId();

            List<String> list = baseMapper.getShopWorkerBy(shopId);
            if (null == list || list.size() == 0) return;

            for (String openId : list) {
                if (StringUtils.isEmpty(openId)) continue;

                List<WxMaSubscribeMessage.Data> dataList = new ArrayList<>();
                dataList.add(new WxMaSubscribeMessage.Data("character_string1", orderId));
                dataList.add(new WxMaSubscribeMessage.Data("thing5", "您有新的十公里订单已核销，请注意查收"));

                try {
                    wxUtils.sendMessage(templateId, openId, page, dataList);
                } catch (Exception e) {
                    e.printStackTrace();
                    log.error("发送订单核销消息失败: {}", e.getMessage());
                } finally {
                    try {
                        Thread.sleep(1000);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
